#include <avr/io.h>
#include <avr/interrupt.h>
#include "keyb.h"
#include "lcd.h"
#include "usart.h"

char buffer[20];
uint8_t bufferIndex = 0;
volatile uint8_t newCommand = 0;

int main(void) {
	DDRD = _BV(DDD2);
	keybInit();
	lcdInit();
	usartInit();
	usartSendString("Test");
	sei();
	//wcz przerwanie
	UCSRB |= _BV(RXCIE);
	while (1) {
		if (readKeyboard()) {
			uint8_t key = getKey();
			if (key < 10) usartSendByte('0' + key);
			if (key == 16) usartSendString("\r\n");
		}
		if (newCommand) {
			newCommand = 0;
			switch (buffer[0]) {
				case 'a':
					PORTD |= _BV(PD2);
					break;
				case 'b':
					PORTD &= ~_BV(PD2);
					break;
				case 't':
					for (uint8_t i = 0; i < buffer[1]; i++) {
						lcdWriteData(buffer[i + 2]);
					}
				break;
			}
		}
	}
}

uint8_t computeCRC8(uint8_t inData, uint16_t seed) {
	uint8_t bitsLeft;
	uint8_t temp;

	for (bitsLeft = 8; bitsLeft > 0; bitsLeft--) {
		temp = ((seed ^ inData) & 0x01);
		if (temp == 0) {
			seed >>= 1;
		}
		else {
			seed ^= 0x18;
			seed >>= 1;
			seed |= 0x80;
		}
		inData >>= 1;
	}
	return seed;
}

ISR(USART_RXC_vect) {
	uint8_t value = UDR;
	if (bufferIndex < sizeof(buffer)) {
		buffer[bufferIndex++] = value;
	}
	if (bufferIndex > 1) {
		uint8_t length = buffer[1];
		if (bufferIndex == length + 2) {
			uint8_t crc = 0;
			for (uint8_t i = 0; i < length + 1; i++) crc = computeCRC8(buffer[i], crc);
			buffer[bufferIndex] = 0;
			bufferIndex = 0;
			if (crc == buffer[bufferIndex - 1]) {
				newCommand = 1;
			}
		}
	}
}
